var MongoClient = require("mongodb").MongoClient;
var ObjectId = require("mongodb").ObjectID;

// Connection URL
const url = "mongodb://localhost:27017";

// Database Name
const dbName = "test";

exports.connexionMongo = async () => {
  try {
    let client = await MongoClient.connect(url, { useNewUrlParser: true });
    let db = await client.db(dbName);
    return true;
  } catch (error) {
    console.log(error.msg);
    return false;
  }
};

exports.countRestaurants = async (name) => {
  let client = await MongoClient.connect(url, { useNewUrlParser: true });
  let db = await client.db(dbName);
  let count = -1;

  if (name == "") {
    count = await db.collection("restaurants").countDocuments();
  } else {
    let query = {
      name: { $regex: ".*" + name + ".*", $options: "i" }
    };
    count = await db
      .collection("restaurants")
      .find(query)
      .countDocuments();
  }

  return count;
};

exports.findRestaurants = async (page, pagesize, name) => {
  let client = await MongoClient.connect(url, { useNewUrlParser: true });
  let db = await client.db(dbName);
  let restaurants, count;
  console.log("name = " + name)
  if (name == "") {
    restaurants = await db
      .collection("restaurants")
      .find()
      .skip(page * pagesize)
      .limit(pagesize)
      .toArray();

    count = await db.collection("restaurants").countDocuments();
  } else {
    let query = {
      name: { $regex: ".*" + name + ".*", $options: "i" }
    };
    restaurants = await db
      .collection("restaurants")
      .find(query)
      .skip(page * pagesize)
      .limit(pagesize)
      .toArray();

    count = await db
      .collection("restaurants")
      .find(query)
      .count();
  }
  return { restaurants, count };
};

exports.findRestaurantById = async (id) => {
  let client = await MongoClient.connect(url, { useNewUrlParser: true });
  let db = await client.db(dbName);
  // La requete mongoDB

  let data, reponse;

  try {
    let myquery = { _id: ObjectId(id) };

    data = await db.collection("restaurants").findOne(myquery);
    reponse = {
      succes: true,
      restaurant: data,
      error: null,
      msg: "Details du restaurant envoyés"
    };
  } catch (error) {
    console.log("Restaurant id = " + id + " non trouvé");
    reponse = {
      succes: false,
      restaurant: null,
      error: error.msg,
      msg: "Restaurant id = " + id + " non trouvé"
    };
  }

  return reponse;
};

exports.createRestaurant = async (formData) => {
  let client = await MongoClient.connect(url, { useNewUrlParser: true });
  let db = await client.db(dbName);
  let reponse;

    try {
      let toInsert = {
        name: formData.nom,
        cuisine: formData.cuisine
      };
      console.dir(JSON.stringify(toInsert));
      let {err, insertedId} = await db.collection("restaurants").insertOne(toInsert);
      console.log("++++" + insertedId);
      console.log("nom = " + formData.nom)
      console.log("cuisine = " + formData.cuisine)

      reponse = {
        succes: true,
        result:  insertedId,
        error: null,
        msg: "Ajout réussi " + insertedId
      };
      console.log("ajout reussi")
    } catch(error) {
      console.log("error : " + error.msg)
      reponse = {
        succes: false,
        error: error.msg,
        msg: "Problème à l'insertion"
      };
    }
    return reponse;
};


exports.updateRestaurant = async (id, formData) => {
  let client = await MongoClient.connect(url, { useNewUrlParser: true });
  let db = await client.db(dbName);
  let reponse, result;

    try {
      let myquery = { _id: ObjectId(id) };
      let newvalues = {
        name: formData.nom,
        cuisine: formData.cuisine
      };
      result = await db.collection("restaurants").replaceOne(myquery, newvalues);
      reponse = {
        succes: true,
        result: result,
        error: null,
        msg: "Modification réussie " + result
      };
    } catch(error) {
      reponse = {
        succes: false,
        error: error.msg,
        msg: "Problème à la modification du restaurant id = " + id
      };
    }
    return reponse;   
};

exports.deleteRestaurant = async (id) => {
  let client = await MongoClient.connect(url, { useNewUrlParser: true });
  let db = await client.db(dbName);
  let reponse, result;
  try {
    let myquery = { _id: ObjectId(id) };

    result = await db.collection("restaurants").deleteOne(myquery);
    reponse = {
      succes: true,
      result: result,
      error: null,
      msg: "Suppression réussie " + result
    };
  } catch(error) {
    reponse = {
      succes: false,
      error: error.msg,
      msg: "Problème lors de la suppression"
    }
  }
  return reponse;
};
