window.onload = init;

function init() {
    // Decommenter si vous utilisez vue-material
    Vue.use(VueMaterial.default);

    // Un composant Restaurant
    Vue.component('app-restaurant', {
        template:`
            <tr>
              <td>{{nom}}, de {{ville}}</td>
              <td>{{cuisine}}</td>
              <td>
                  <button 
                      v-on:click="supprimer(index)"
                  >
                     Supprimer
                  </button>
              </td>
            </tr>
        `,
        props: {
            nom:{
                type:String,
                required:true
            },
            ville:"",
            cuisine:"",
            index: {
                required:true
            }
        },
        data() {
            return {
            }
        },
        methods: {
            supprimer(index) {
                console.log("supprimer " + this.index);
                // on envoie à d'éventuels écouteurs (autres composants)
                // un message custom "restaurantclicked"
                this.$emit('restaurantclicked', this.index);
            }
        }

    });

    new Vue({
        el: "#app",
        data: {
            nomDuProf: "Michel Buffa",
            nom:"",
            cuisine:"",
            restaurants: [],
            message:"",
            apiURL: "/api/restaurants",
            page :0,
            pagesize: 5,
            nomRecherche : "",
            nbRestaurants:0,
            nbPagesDeResultats:0,
            // si bootstrap table
            //fields: ['name', 'cuisine'],
        },
        mounted() {
            // appelé avant affichage
            console.log("AVANT AFFICHAGE");
            this.getDataFromServer();
        },
        methods: {
            getDataFromServer() {
                this.message = "Je requête le serveur..."
                console.log("J'envoie requête AJAX sur WebService pour chercher des données...");
                let url = this.apiURL 
                + "?page=" + this.page 
                + "&pagesize=" + this.pagesize 
                + "&name=" + this.nomRecherche;

                fetch(url)
                .then(reponseJSON => {
                    return reponseJSON.json()
                }).then(reponseJS => {
                    this.restaurants = reponseJS.data;

                    console.log("données récupérées...")
                    this.message = reponseJS.msg;
                    this.nbRestaurants = reponseJS.count;
                    this.nbPagesDeResultats = Math.ceil(this.nbRestaurants/this.pagesize);
                    // exécuté au bout de 1s
                    /*
                    setTimeout(() => {
                        this.message = "";
                    }, 1000);
                    */
                })
            },
            supprimer(id) {
                console.log("Supprimer restaurant id=" + id);
                // on envoie une requête sur le serveur
                let url = this.apiURL + "/" + id;

                fetch(url, {
                    method:"DELETE"
                })
                .then(reponseJSON => {
                    return reponseJSON.json()
                }).then(reponseJS => {
                    console.log("Restaurant supprime id = " + reponseJS.msg)
                    this.message = reponseJS.msg;
                    // on reaffiche les restaurants car celui qu'on 
                    // a supprimé est dans la page
                    this.getDataFromServer();
                });
            },
            ajouterRestaurant() {
                console.log("Ajout restaurant nom " 
                + this.nom + " cuisine = " + this.cuisine);

                // On construit un objet FormData qui va
                // contenir des données identiques à 
                // celles d'un formulaire MULTIPART
                let formData = new FormData(/* eventuellement un form  */);
                formData.append("nom", this.nom);
                formData.append("cuisine",  this.cuisine);

                // On envoie un POST en Ajax au serveur
                fetch(this.apiURL, {
                    method:"POST",
                    body:formData
                }).then(reponseJSON => {
                    return reponseJSON.json();
                }).then(reponseJS => {
                    this.message = reponseJS.msg;
                    this.getDataFromServer();
                })
                // on vide le formulaire
                this.nom = "";
                this.cuisine = "";
            },
            getColor(index) {
                if(index % 2) return "red";
                else return "green";
            },
            pageSuivante() {
                this.page++;
                this.getDataFromServer();
            },
            pagePrecedente() {
                this.page--;
                this.getDataFromServer();
            },
            rechercheParNom() {
                this.getDataFromServer();
            }
        }
    });
};