const express = require("express");
const app = express();
const server = require("http").Server(app);

// pour les formulaires multiparts
var multer = require("multer");
var multerData = multer();

const port = process.env.PORT || 8080;

const mongoDBModule = require("./app_modules/crud-mongo");

// Pour les formulaires standards
const bodyParser = require("body-parser");
// pour les formulaires multiparts
var multer = require("multer");
var multerData = multer();

// Cette ligne indique le répertoire qui contient
// les fichiers statiques: html, css, js, images etc.
app.use(express.static(__dirname + "/public"));
// Paramètres standards du modyle bodyParser
// qui sert à récupérer des paramètres reçus
// par ex, par l'envoi d'un formulaire "standard"
app.use(bodyParser.urlencoded({ extended: false }));
app.use(bodyParser.json());

// Pour le cross domain (autorise des clients de domaines différents)
app.use((req, res, next) => {
  res.header("Access-Control-Allow-Origin", "*");
  res.header(
    "Access-Control-Allow-Headers",
    "Origin, X-Requested-With, Content-Type, Accept"
  );
  res.header("Access-Control-Allow-Methods", "POST, GET, OPTIONS, PUT, DELETE");

  next();
});

// Lance le serveur avec express
server.listen(port);
console.log("Serveur lancé sur le port : " + port);

//------------------
// ROUTES
//------------------
// Utile pour indiquer la home page, dans le cas
// ou il ne s'agit pas de public/index.html
app.get("/", function(req, res) {
  res.sendFile(__dirname + "/public/index.html");
});

app.get("/MyTableVue", function(req, res) {
  res.sendFile(__dirname + "/public/TableVue.html");
});

// Ici des routes en :
// http GET (pour récupérer des données)
// http POST : pour insérer des données
// http PUT pour modifier des données
// http DELETE pour supprimer des données

//----------------------------------------------
// Ces routes forment l'API de l'application !!
//----------------------------------------------

// Test de la connexion à la base
app.get("/api/connection", async (req, res) => {
  // Pour le moment on simule, mais après on devra
  // réellement se connecte à la base de données
  // et renvoyer une valeur pour dire si tout est ok
  let connectionOk = await mongoDBModule.connexionMongo();

  if (connectionOk) {
    console.log("connexion etablie");
    reponse = {
      msg: "connexion établie"
    };
  } else {
    console.log("erreur connexion");
    reponse = {
      msg: "erreur de connexion"
    };
  }
  res.send(JSON.stringify(reponse));
});

app.get("/api/restaurants/count", async (req, res) => {
  // Pour le moment on simule, mais après on devra
  // réellement se connecte à la base de données
  // et renvoyer une valeur pour dire si tout est ok
  let name = req.query.name || "";

  count = await mongoDBModule.countRestaurants(name);
  var objdData = {
    msg: "restaurants count",
    data: count
  };
  res.send(JSON.stringify(objdData));
});

// On va récupérer des restaurants par un GET (standard REST)
// cette fonction d'API peut accepter des paramètres
// pagesize = nombre de restaurants par page
// page = no de la page
// Oui, on va faire de la pagination, pour afficher
// par exemple les restaurants 10 par 10
app.get("/api/restaurants", async (req, res) => {
  // Si présent on prend la valeur du param, sinon 1
  let page = parseInt(req.query.page || 1);
  // idem si present on prend la valeur, sinon 10
  let pagesize = parseInt(req.query.pagesize || 10);

  let name = req.query.name || "";

  let { restaurants, count } = await mongoDBModule.findRestaurants(
    page,
    pagesize,
    name
  );
  console.log(
    "count = " + count + " restaurants.length = " + restaurants.length
  );
  //console.log("count = " + count + " data.length : " + data.length);
  var objdData = {
    msg: "restaurant recherchés avec succès",
    data: restaurants,
    count: count
  };
  res.send(JSON.stringify(objdData));
});

// Récupération d'un seul restaurant par son id
app.get("/api/restaurants/:id", async (req, res) => {
  var id = req.params.id;

  let reponse = await mongoDBModule.findRestaurantById(id); 
  console.log("reponse = " + reponse);
  res.send(JSON.stringify(reponse));
});

// Creation d'un restaurant par envoi d'un formulaire
// On fera l'insert par un POST, c'est le standard REST
app.post("/api/restaurants", multerData.fields([]), async (req, res) => {
  // On supposera qu'on ajoutera un restaurant en
  // donnant son nom et sa cuisine. On va donc
  // recuperer les données du formulaire d'envoi
  // les params sont dans req.body même si le formulaire
  // est envoyé en multipart

  console.log("INSERT nom = "+ req.body.nom);
  let reponse = await mongoDBModule.createRestaurant(req.body)
    res.send(JSON.stringify(reponse));
});

// Modification d'un restaurant, on fera l'update par
// une requête http PUT, c'est le standard REST
app.put("/api/restaurants/:id", multerData.fields([]), async (req, res) => {
  var id = req.params.id;

  let reponse = await mongoDBModule.updateRestaurant(id, req.body);
    res.send(JSON.stringify(reponse));

});

// Suppression d'un restaurant
// On fera la suppression par une requête http DELETE
// c'est le standard REST
app.delete("/api/restaurants/:id", async (req, res) => {
  var id = req.params.id;

  let reponse = await mongoDBModule.deleteRestaurant(id); 
    res.send(JSON.stringify(reponse));
});
